@@ -2,7 +2,7 @@
 /* file analysis.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2010 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
     Theresienstr. 39
     D-80333 Muenchen, Germany
 
-Email   forster@rz.mathematik.uni-muenchen.de
+Email   forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 /*
@@ -33,8 +33,9 @@ Email   forster@rz.mathematik.uni-muenchen.de
 ** transcendental functions
 **
 ** date of last change
-** 95-02-21
-** 97-04-14:    changed inipi
+** 1995-02-21
+** 1997-04-14:    changed inipi
+** 2010-01-23:    bugfix in sin0 and cos0 with floatprec > 2048
 */
 
 #include "common.h"
@@ -829,7 +830,16 @@ word2 *x, *z, *hilf;
     for(i=2; m>0; i+=2) {
         m = multfix(prec,x2,m2,temp,m,temp1,hilf);
         cpyarr(temp1,m,temp);
-        m = divarr(temp,m,i*(i+1),hilf);
+        if(i<=255)
+            m = divarr(temp,m,i*(i+1),hilf);
+        else {
+#ifdef M64_32
+            m = div4arr(temp,m,i*(i+1),hilf);
+#else
+            m = divarr(temp,m,i,hilf);
+            m = divarr(temp,m,i+1,hilf);
+#endif
+        }
         if(i & 2)
             len = subarr(z,len,temp,m);
         else
@@ -863,7 +873,16 @@ word2 *x, *z, *hilf;
     cpyarr(x2,m2,temp);
     m = m2;
     for(i=1; ; i+=2) {
-        m = divarr(temp,m,i*(i+1),hilf);
+        if(i<=255)
+            m = divarr(temp,m,i*(i+1),hilf);
+        else {
+#ifdef M64_32
+            m = div4arr(temp,m,i*(i+1),hilf);
+#else
+            m = divarr(temp,m,i,hilf);
+            m = divarr(temp,m,i+1,hilf);
+#endif
+        }
         if(i & 2)
             len = addarr(z,len,temp,m);
         else
