@@ -53,7 +53,6 @@ Email   forster@mathematik.uni-muenchen.de
 PUBLIC void iniarith    _((void));
 PUBLIC truc addints     _((truc *ptr, int minflg));
 PUBLIC truc scalintvec  _((truc *ptr1, truc *ptr2));
-PUBLIC truc Gvecmod     _((int flg));
 PUBLIC unsigned random2 _((unsigned u));
 PUBLIC unsigned random4 _((unsigned u));
 PUBLIC int cmpnums      _((truc *ptr1, truc *ptr2, int type));
@@ -99,6 +98,7 @@ PRIVATE truc Fdivf      _((void));
 PRIVATE truc Fdiv       _((void));
 PRIVATE truc Fdivide    _((void));
 PRIVATE truc Fmod       _((void));
+PRIVATE truc Gvecmod     _((int flg));
 PRIVATE truc divide     _((truc *ptr, int tflag));
 PRIVATE truc modout     _((truc *ptr));
 PRIVATE truc divfloats  _((truc *ptr));
@@ -709,6 +709,7 @@ truc *ptr1, *ptr2;
 /*---------------------------------------------------------*/
 /*
 ** Multiplies int vector given in *ptr2 by scalar in *ptr1
+** uses AriBuf and AriScratch
 */
 PUBLIC truc scalintvec(ptr1,ptr2)
 truc *ptr1, *ptr2;
@@ -1223,7 +1224,7 @@ truc *ptr;
 ** mod out (destructively) vector given in argStkPtr[-1]
 ** by fixnum resp. bignum in argStkPtr[0]
 */
-PUBLIC truc Gvecmod(flg)
+PRIVATE truc Gvecmod(flg)
 int flg;    /* flg == fFIXNUM or fBIGNUM */
 {
     truc *ptr;
