@@ -77,6 +77,7 @@ PUBLIC void iniaritz    _((void));
 
 PUBLIC truc gf2nzero, gf2none;
 PUBLIC truc gf2nintsym, gf2n_sym;
+PUBLIC truc znXmultsym, znXsqsym, znXddivsym, znXdivsym, znXmodsym;
 PUBLIC truc polmultsym, polNmultsym;
 PUBLIC truc polmodsym, polNmodsym, poldivsym, polNdivsym;
 PUBLIC truc addgf2ns    _((truc *ptr));
@@ -91,14 +92,25 @@ PUBLIC unsigned fp_sqrt    _((unsigned p, unsigned a));
 
 
 PRIVATE truc Fgf2nint       _((void));
+PRIVATE truc FznXmult       _((void));
+PRIVATE truc FznXsquare     _((void));
+PRIVATE truc FznXdivide     _((void));
+PRIVATE truc FznXdiv        _((void));
+PRIVATE truc FznXmod        _((void));
 PRIVATE truc Fpolmult       _((void));
 PRIVATE truc FpolNmult      _((void));
 PRIVATE truc Fpolmod        _((void));
 PRIVATE truc FpolNmod       _((void));
 PRIVATE truc Fpoldiv        _((void));
 PRIVATE truc FpolNdiv       _((void));
+PRIVATE truc znXdivmodsymb  _((int mode));
+PRIVATE int chkznXmultargs  _((truc sym, truc *argptr));
 PRIVATE int chkpolmultargs  _((truc sym, truc *argptr));
 PRIVATE int chkpoldivargs   _((truc sym, truc *argptr));
+PRIVATE truc znXpolmult    _((truc *mptr, truc *argptr));
+PRIVATE truc znXpolsquare   _((truc *mptr, truc *argptr));
+PRIVATE truc znXpoldiv     _((truc *mptr, truc *argptr, int mode));
+PRIVATE truc znXpolmod     _((truc *mptr, truc *argptr));
 PRIVATE truc multintpols    _((truc *argptr, int mode));
 PRIVATE truc modintpols     _((truc *argptr, int mode));
 
@@ -170,6 +182,7 @@ PRIVATE truc Fgg2   _((void));
 #endif
 
 #define GF2XARITH
+/* #define ZnXARITH */
 /*------------------------------------------------------------------*/
 PUBLIC void iniaritz()
 {
@@ -198,6 +211,13 @@ PUBLIC void iniaritz()
     gf2Xprimsym = newsymsig("gf2X_primetest",sFBINARY,(wtruc)Fgf2Xprimtest, s_1);
 #endif /* GF2XARITH */
 
+#ifdef ZnXARITH
+    znXmultsym = newsymsig("ZnX_mult",  sFBINARY, (wtruc)FznXmult, s_3);
+    znXsqsym   = newsymsig("ZnX_square",sFBINARY, (wtruc)FznXsquare, s_2);
+    znXddivsym = newsymsig("ZnX_divide",sFBINARY, (wtruc)FznXdivide, s_3);
+    znXdivsym  = newsymsig("ZnX_div",   sFBINARY, (wtruc)FznXdiv, s_3);
+    znXmodsym  = newsymsig("ZnX_mod",   sFBINARY, (wtruc)FznXmod, s_3);
+#endif
 #ifdef POLYARITH
     polmultsym = newsymsig("pol_mult",  sFBINARY, (wtruc)Fpolmult, s_2);
     polNmultsym= newintsym("pol_mult() mod", sFBINARY,(wtruc)FpolNmult);
@@ -875,6 +895,74 @@ PRIVATE truc Fpolmult()
     }
 }
 /*------------------------------------------------------------------*/
+PRIVATE truc FznXmult()
+{
+    int type;
+
+    if(chkintnz(znXmultsym,argStkPtr-2) == aERROR)
+        return brkerr();
+    type = chkznXmultargs(znXmultsym,argStkPtr-1);
+    if(type == aERROR)
+        return brkerr(); 
+    return znXpolmult(argStkPtr-2,argStkPtr-1);
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc FznXsquare()
+{
+    int type;
+
+    if(chkintnz(znXsqsym,argStkPtr-1) == aERROR)
+        return brkerr();
+    type = *FLAGPTR(argStkPtr);
+    if(type != fVECTOR) {
+        error(znXsqsym,err_vect,*argStkPtr);
+        return brkerr();
+    }
+    if(chkintvec(znXsqsym,argStkPtr) == aERROR)
+        return brkerr();
+
+    return znXpolsquare(argStkPtr-1,argStkPtr); 
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc FznXdivide()
+{
+    int type, mode;
+
+    if(chkintnz(znXdivsym,argStkPtr-2) == aERROR)
+        return brkerr();
+    type = chkznXmultargs(znXdivsym,argStkPtr-1);
+    if(type == aERROR)
+        return brkerr(); 
+    mode = DDIVFLAG;
+    return znXpoldiv(argStkPtr-2,argStkPtr-1,mode);
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc FznXdiv()
+{
+    int type, mode;
+
+    if(chkintnz(znXdivsym,argStkPtr-2) == aERROR)
+        return brkerr();
+    type = chkznXmultargs(znXdivsym,argStkPtr-1);
+    if(type == aERROR)
+        return brkerr(); 
+    mode = DIVFLAG;
+    return znXpoldiv(argStkPtr-2,argStkPtr-1,mode);
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc FznXmod()
+{
+    int type, mode;
+
+    if(chkintnz(znXmodsym,argStkPtr-2) == aERROR)
+        return brkerr();
+    type = chkznXmultargs(znXdivsym,argStkPtr-1);
+    if(type == aERROR)
+        return brkerr(); 
+    mode = MODFLAG;
+    return znXpoldiv(argStkPtr-2,argStkPtr-1,mode);
+}
+/*------------------------------------------------------------------*/
 PRIVATE truc FpolNmult()
 {
     int type;
@@ -885,6 +973,406 @@ PRIVATE truc FpolNmult()
     return multintpols(argStkPtr-2,MODNFLAG);
 }
 /*------------------------------------------------------------------*/
+PRIVATE truc znXpolsquare(mptr,argptr)
+truc *mptr; 
+truc *argptr;
+{
+    truc *ptr1;
+    truc *workptr, *ptr, *wptr;
+    struct vector *vecptr;
+    truc obj;
+    word2 *x, *y, *zz, *aa, *hilf;
+    int len, len1, k, i, j0, j1;
+    int n1, n2, n3, n, m, sign, sign1, sign2, sign3;
+    unsigned mlen, offshilf;
+
+    len1 = *VECLENPTR(argptr);
+    ptr1 = argptr;
+    if(len1 == 0)
+        return mkvect0(0);
+
+    /* now len1 >= 1 */
+    workptr = workStkPtr+1;
+    if(!WORKspace(len1)) {
+        error(znXsqsym,err_memev,voidsym);
+        return brkerr();
+    }
+    mlen = aribufSize/3 - 6;
+    ptr = VECTORPTR(ptr1);
+    wptr = workptr;
+    for(i=0; i<len1; i++) {      /* vector stored in workptr[] */
+        if(*FLAGPTR(ptr) == fBIGNUM && *BIGLENPTR(ptr) >= mlen)
+            goto ovflexit;
+        *wptr++ = *ptr++;
+    }
+    len = 2*len1 - 1;
+    obj = mkvect0(len);
+    WORKpush(obj);
+
+    zz = AriBuf;
+    n3 = bigretr(mptr,zz,&sign3);
+    if(n3 >= mlen)
+        goto ovflexit;
+    aa = AriBuf + ((mlen + 6) & 0xFFFE);
+    offshilf = (scrbufSize/2) & 0xFFFE;
+    for(k=0; k<len; k++) {
+        hilf = AriScratch + offshilf;
+        n = 0; sign = 0;
+        j0 = (k < len1 ? 0 : k+1-len1);
+        j1 = (k+1)/2;
+        for(i=j0; i<j1; i++) {
+            n1 = bigref(workptr+i,&x,&sign1);
+            n2 = bigref(workptr+(k-i),&y,&sign2);
+            m = multbig(x,n1,y,n2,AriScratch,hilf);
+            sign1 = (sign1 == sign2 ? 0 : MINUSBYTE);
+            n = addsarr(aa,n,sign,AriScratch,m,sign1,&sign);
+        }
+        n = shlarr(aa,n,1);
+        if ((k&1) == 0) {
+            n1 = bigref(workptr+(k/2),&x,&sign1);
+            m = multbig(x,n1,x,n1,AriScratch,hilf);
+            sign1 = 0;
+            n = addsarr(aa,n,sign,AriScratch,m,sign1,&sign);
+        }
+        n = modbig(aa,n,zz,n3,hilf);
+        if(n && (sign != sign3)) {
+            n = sub1arr(aa,n,zz,n3);
+            sign = sign3;
+        }
+        obj = mkint(sign,aa,n);
+        *(VECTORPTR(workStkPtr) + k) = obj;
+    }
+    vecptr = VECSTRUCTPTR(workStkPtr);
+    ptr = &(vecptr->ele0) + len - 1;
+    while(len > 0 && *ptr-- == zero)
+        len--;
+    vecptr->len = len;
+    obj = WORKretr();
+    workStkPtr = workptr-1;
+    return obj;
+  ovflexit:
+    workStkPtr = workptr-1;
+    error(znXsqsym,err_ovfl,voidsym);
+    return(brkerr());
+}
+/*------------------------------------------------------------------*/
+/*
+** multiplies two integer polynomials given by argptr[0] and argptr[1]
+** and mods them out modulo *mptr
+*/
+PRIVATE truc znXpolmult(mptr,argptr)
+truc *mptr;
+truc *argptr;
+{
+    truc *ptr1, *ptr2;
+    truc *workptr, *w2ptr, *ptr, *wptr;
+    struct vector *vecptr;
+    truc obj;
+    word2 *x, *y, *zz, *aa, *hilf;
+    int len, len1, len2, k, i, j0, j1;
+    int n1, n2, n3, n, m, sign, sign1, sign2, sign3;
+    unsigned mlen, offshilf;
+
+    len = *VECLENPTR(argptr);
+    len2 = *VECLENPTR(argptr+1);
+    if(len >= len2) {
+        len1 = len;
+        ptr1 = argptr;
+        ptr2 = argptr + 1;
+    }
+    else {
+        len1 = len2;
+        len2 = len;
+        ptr1 = argptr + 1;
+        ptr2 = argptr;
+    }
+    /* now len1 >= len2, lenk = length of vector *ptrk */
+    if(len2 == 0)
+        return mkvect0(0);
+
+    /* now len2 >= 1 */
+    workptr = workStkPtr+1;
+    if(!WORKspace(len1+len2)) {
+        error(znXmultsym,err_memev,voidsym);
+        return brkerr();
+    }
+    mlen = aribufSize/3 - 6;
+    ptr = VECTORPTR(ptr1);
+    wptr = workptr;
+    for(i=0; i<len1; i++) {      /* first vector stored in workptr[] */
+        if(*FLAGPTR(ptr) == fBIGNUM && *BIGLENPTR(ptr) >= mlen)
+            goto ovflexit;
+        *wptr++ = *ptr++;
+    }
+    ptr = VECTORPTR(ptr2);
+    wptr = w2ptr = workptr + len1;
+    for(k=0; k<len2; k++) {     /* store second polynomial */
+        if(*FLAGPTR(ptr) == fBIGNUM && *BIGLENPTR(ptr) >= mlen)
+            goto ovflexit;
+        *wptr++ = *ptr++;
+    }
+    len = len1 + len2 - 1;
+    obj = mkvect0(len);
+    WORKpush(obj);
+
+    zz = AriBuf;
+    n3 = bigretr(mptr,zz,&sign3);
+    if(n3 >= mlen)
+        goto ovflexit;
+    aa = AriBuf + ((mlen + 6) & 0xFFFE);
+    offshilf = (scrbufSize/2) & 0xFFFE;
+    for(k=0; k<len; k++) {
+        hilf = AriScratch + offshilf;
+        n = 0; sign = 0;
+        j0 = (k < len2 ? 0 : k+1-len2);
+        j1 = (k < len1 ? k : len1-1);
+        for(i=j0; i<=j1; i++) {
+            n1 = bigref(workptr+i,&x,&sign1);
+            n2 = bigref(w2ptr+(k-i),&y,&sign2);
+            m = multbig(x,n1,y,n2,AriScratch,hilf);
+            sign1 = (sign1 == sign2 ? 0 : MINUSBYTE);
+            n = addsarr(aa,n,sign,AriScratch,m,sign1,&sign);
+        }
+        n = modbig(aa,n,zz,n3,hilf);
+        if(n && (sign != sign3)) {
+            n = sub1arr(aa,n,zz,n3);
+            sign = sign3;
+        }
+        obj = mkint(sign,aa,n);
+        *(VECTORPTR(workStkPtr) + k) = obj;
+    }
+    vecptr = VECSTRUCTPTR(workStkPtr);
+    ptr = &(vecptr->ele0) + len - 1;
+    while(len > 0 && *ptr-- == zero)
+        len--;
+    vecptr->len = len;
+    obj = WORKretr();
+    workStkPtr = workptr-1;
+    return obj;
+  ovflexit:
+    workStkPtr = workptr-1;
+    error(znXmultsym,err_ovfl,voidsym);
+    return(brkerr());
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc znXdivmodsymb(mode)
+int mode;
+{
+    if (mode == MODFLAG)
+        return znXmodsym;
+    else if (mode == DIVFLAG)
+        return znXdivsym;
+    else if (mode == DDIVFLAG)
+        return znXddivsym;
+    else
+        return voidsym;
+}
+/*------------------------------------------------------------------*/
+/*
+** division of integer polynomial given by argptr[0] by argptr[1],
+** modded out modulo *mptr
+*/
+PRIVATE truc znXpoldiv(mptr,argptr,mode)
+truc *mptr;
+truc *argptr;
+int mode;
+{
+    truc *workptr, *w1ptr, *w2ptr, *leadptr, *ptr1, *ptr2;
+    truc *vptr;
+    truc obj;
+    word2 *yy, *zz, *aa, *bb, *hilf;
+    unsigned mlen, offsbb, offshilf;
+    int sign, sign1, sign2, sign3, signL;
+    int j, k, m, m1, m2, n, len1, len2, len3, len4;
+    int leadcorr = 0;
+    int mode1;
+
+    len1 = *VECLENPTR(argptr);
+    len2 = *VECLENPTR(argptr+1);
+    if(!len2) {
+        error(znXdivmodsymb(mode),err_div,voidsym);
+        return brkerr();
+    }
+
+    if(len2 > len1) {
+        if (mode == MODFLAG)
+            return *argptr;
+        else
+            obj = mkvect0(0);
+        if (mode == DIVFLAG)
+            return obj;
+        else { /* mode == DDIVFLAG */
+            WORKpush(obj);
+            obj = mkvect0(2);
+            vptr = VECTOR(obj);
+            vptr[0] = WORKretr();
+            vptr[1] = *argptr;
+            return obj;
+        }
+    }
+
+    mlen = aribufSize/3 - 6;
+    if(*FLAGPTR(mptr) == fBIGNUM && *BIGLENPTR(mptr) >= mlen)
+        goto ovflexit;
+    if(*SIGNPTR(mptr) == MINUSBYTE || *mptr == zero) {
+        error(znXdivmodsymb(mode),err_pint,*mptr);
+        return brkerr();        
+    }
+
+    workptr = workStkPtr+1;
+    if(!WORKspace(len1 + len2)) {
+        error(znXdivmodsymb(mode),err_memev,voidsym);
+        return brkerr();
+    }
+    w2ptr = workptr + len1;
+
+    offsbb = (mlen + 6) & 0xFFFE;
+    offshilf = (scrbufSize/2) & 0xFFFE;
+
+    ptr1 = workptr;
+    ptr2 = VECTORPTR(argptr);
+    for(k=0; k<len1; k++) {     /* store first polynomial */
+        if(*FLAGPTR(ptr2) == fBIGNUM && *BIGLENPTR(ptr2) >= mlen)
+            goto ovflexit;
+        *ptr1++ = *ptr2++;
+    }
+    ptr1 = w2ptr;
+    ptr2 = VECTORPTR(argptr+1);
+    for(k=0; k<len2; k++) {     /* store second polynomial */
+        if(*FLAGPTR(ptr2) == fBIGNUM && *BIGLENPTR(ptr2) >= mlen)
+            goto ovflexit;
+        *ptr1++ = *ptr2++;
+    }
+    aa = AriBuf;
+    bb = AriBuf + offsbb;
+
+    if(w2ptr[len2-1] != constone) {
+        leadcorr = 1;
+        leadptr = w2ptr + len2 - 1;
+        n = bigretr(leadptr,bb,&signL);
+        m2 = bigref(mptr,&zz,&sign2);
+        m = modinverse(bb,n,zz,m2,aa,AriScratch);
+        if (m==0) {
+            error(znXdivmodsymb(mode),
+                "leading coeff must be invertible mod n",w2ptr[len2-1]);
+            return brkerr();
+        }
+        obj = mkint(signL,aa,m);
+        *leadptr = obj;
+        for(k=0; k<len2-1; k++) {
+            n = bigref(leadptr,&zz,&signL);
+            m = bigref(w2ptr+k, &yy, &sign1);
+            n = multbig(yy,m,zz,n,bb,AriScratch);
+            sign = (signL == sign1 ? 0 : MINUSBYTE);
+            m2 = bigref(mptr,&zz,&sign2);
+            n = modbig(bb,n,zz,m2,AriScratch);
+            obj = mkint(sign,bb,n);
+            w2ptr[k] = obj;
+        }
+    }
+
+    len3 = len1 - len2 + 1;
+    for(k=len3-1; k>=0; k--) {
+        n = bigretr(workptr+len2-1+k,aa,&sign);
+        m2 = bigref(mptr,&zz,&sign2);
+        n = modbig(aa,n,zz,m2,AriScratch);
+        if(!n)
+            continue;
+        for(j=0; j<=len2-2; j++) {
+            m = bigref(w2ptr+j,&yy,&sign1);
+            if(!m)
+                continue;
+            m = multbig(aa,n,yy,m,AriScratch,AriScratch+offshilf);
+            sign1 = (sign == sign1 ? MINUSBYTE : 0);
+            /* sign of the negative product */
+            m1 = bigretr(workptr+k+j, bb, &sign2);
+            m = addsarr(bb,m1,sign2,AriScratch,m,sign1,&sign3);
+            obj = mkint(sign3,bb,m);
+            workptr[k+j] = obj;
+        }
+    }
+    if (mode & MODFLAG) {
+        len4 = len2 - 1;
+        for(k=0; k<len4; k++) {
+            n = bigretr(workptr+k,aa,&sign);
+            m2 = bigref(mptr,&zz,&sign2);
+            n = modbig(aa,n,zz,m2,AriScratch);
+            if(n && (sign != sign2)) {
+                n = sub1arr(aa,n,zz,m2);
+                sign = sign2;
+            }
+            obj = mkint(sign,aa,n);
+            workptr[k] = obj;
+        }
+        while(len4>0 && workptr[len4-1] == zero)
+            len4--;
+        obj = mkvect0(len4);
+        ptr1 = VECTOR(obj);
+        ptr2 = workptr;
+        for(k=0; k<len4; k++)
+            *ptr1++ = *ptr2++;
+        WORKpush(obj);
+    }
+    if (mode & DIVFLAG) {
+        w1ptr = workptr + len2 - 1;
+        if (leadcorr) {
+            m = bigretr(leadptr,aa,&signL);
+            for(k=0; k<len3; k++) {
+                n = bigref(w1ptr+k,&yy,&sign);
+                if (!n)
+                    continue;
+                n = multbig(yy,n,aa,m,bb,AriScratch);
+                sign = (signL == sign ? 0 : MINUSBYTE);
+                m2 = bigref(mptr,&zz,&sign2);
+                n = modbig(bb,n,zz,m2,AriScratch);
+                if(n && (sign != sign2)) {
+                    n = sub1arr(bb,n,zz,m2);
+                    sign = sign2;
+                }
+                obj = mkint(sign,bb,n);
+                w1ptr[k] = obj;
+            }
+        }
+        else {
+            for(k=0; k<len3; k++) {
+                n = bigretr(w1ptr+k,aa,&sign);
+                m2 = bigref(mptr,&zz,&sign2);
+                n = modbig(aa,n,zz,m2,AriScratch);
+                if(n && (sign != sign2)) {
+                    n = sub1arr(aa,n,zz,m2);
+                    sign = sign2;
+                }
+                obj = mkint(sign,aa,n);
+                w1ptr[k] = obj;
+            }
+        }
+        while(len3>0 && w1ptr[len3-1] == zero)
+            len3--;
+        obj = mkvect0(len3);
+        ptr1 = VECTOR(obj);
+        ptr2 = w1ptr;
+        for(k=0; k<len3; k++)
+            *ptr1++ = *ptr2++;
+        WORKpush(obj);
+    }
+    if (mode == DDIVFLAG) {
+        obj = mkvect0(2);
+        vptr = VECTOR(obj);
+        vptr[0] = WORKretr();
+        vptr[1] = WORKretr();
+    }
+    else {
+        obj = WORKretr();
+    }
+    workStkPtr = workptr-1;
+    return obj;
+
+  ovflexit:
+    workStkPtr = workptr-1;
+    error(znXdivmodsymb(mode),err_ovfl,voidsym);
+    return(brkerr());
+}
+/*------------------------------------------------------------------*/
 /*
 ** multiplies two integer polynomials given by argptr[0] and argptr[1]
 */
@@ -1165,6 +1653,28 @@ int mode;
     return(brkerr());
 }
 /*------------------------------------------------------------------*/
+PRIVATE int chkznXmultargs(sym,argptr)
+truc sym;
+truc *argptr;
+{
+    int flg1, flg2;
+    truc *ptr;
+
+    flg1 = *FLAGPTR(argptr);
+    flg2 = *FLAGPTR(argptr+1);
+    if(flg1 != fVECTOR || flg2 != fVECTOR) {
+        ptr = (flg1 == fVECTOR ? argptr+1 : argptr);
+        return error(sym,err_vect,*ptr);
+    }
+    flg1 = chkintvec(sym,argptr);
+    if(flg1 != aERROR)
+        flg2 = chkintvec(sym,argptr+1);
+    if(flg1 == aERROR || flg2 == aERROR)
+        return aERROR;
+
+    return (flg1 >= flg2 ? flg1 : flg2);
+}
+/*------------------------------------------------------------------*/
 PRIVATE int chkpolmultargs(sym,argptr)
 truc sym;
 truc *argptr;
@@ -1748,7 +2258,7 @@ PRIVATE truc Fgf2Xmodpow()
     else if (sign) {
 /******************************/
 /* TODO: negative exponent */
-        error(gf2Xmpowsym,err_pint,argStkPtr[-1]);
+        error(gf2Xmpowsym,err_p0int,argStkPtr[-1]);
         return brkerr();
     }
     len = gf2polmodpow(x,n,y,m,f,flen,AriBuf,AriScratch);
