@@ -1009,6 +1009,8 @@ long *pindex;
 {
     long index;
     int flg, flg1;
+    word2 *z;
+    int sign, n;
 
     flg = *FLAGPTR(ptr);
     if(flg < fVECTLIKE0 && flg > fVECTLIKE1) {
@@ -1023,8 +1025,21 @@ long *pindex;
             index = -index;
         *pindex = index;
     }
-    else {      /* vorlaeufig */
-        error(arr_sym,err_fix,*ptr);
+    else if(flg1 == fBIGNUM) {
+        n = bigref(ptr,&z,&sign);
+        if (n <= 2) {
+            index = big2long(z,n);
+            if(sign) index = -index;
+            *pindex = index;
+        }
+        else {
+            error(arr_sym,err_p4int,*ptr);
+            return(aERROR);
+        }
+    }
+    else {      
+    /* vorlaeufig */
+        error(arr_sym,err_pfix,*ptr);
         return(aERROR);
     }
     return(flg);
