@@ -189,8 +189,9 @@ directory
 #define FPREC_HIGH
 #endif
 
-#define VERSION_STRING "V 1.63, Feb. 2008"
-#define VERSION_NO 163
+#define VERSION_STRING "V 1.64, Jan. 2010"
+#define VERSION_YEAR "2010"
+#define VERSION_NO 164
 
 /*-----------------------------------------------------------------*/
 #ifdef PROTO
@@ -850,7 +851,7 @@ extern char *err_funest, *err_funame, *err_2ident,
         *err_inpf, *err_tinp, *err_binp,
         *err_then, *err_end,
         *err_ovfl, *err_div, *err_2big, *err_float, *err_bool,
-        *err_int, *err_intt, *err_fix, *err_pfix, *err_pint, *err_p4int,
+        *err_int, *err_intt, *err_fix, *err_pfix, *err_pint, *err_p0int, *err_p4int,
         *err_odd, *err_oddprim,
         *err_char, *err_chr, *err_2long, *err_n2long, *err_iovfl,
         *err_num, *err_pnum, *err_p0num, *err_intvar,
@@ -915,7 +916,6 @@ extern unsigned random2 _((unsigned u));
 extern unsigned random4 _((unsigned u));
 extern int cmpnums      _((truc *ptr1, truc *ptr2, int type));
 extern truc scalintvec  _((truc *ptr1, truc *ptr2));
-extern truc Gvecmod     _((int flg));
 
 extern truc integsym, int_sym, realsym;
 extern truc zero, constone, flt0zero;
@@ -956,6 +956,7 @@ extern int showvect  _((FILE *f, word2 *xx, int len));
 /* aritz.c */
 extern void iniaritz    _((void));
 extern truc gf2nzero, gf2none, gf2nintsym, gf2n_sym;
+extern truc znXmultsym, znXsqsym, znXddivsym, znXdivsym, znXmodsym;
 extern truc polmultsym, polNmultsym, polmodsym, polNmodsym,
         poldivsym, polNdivsym;
 extern truc addgf2ns    _((truc *ptr));
