@@ -42,6 +42,7 @@ Email   forster@mathematik.uni-muenchen.de
 ** 1998-12-27:  issepdir()
 ** 2001-06-02:  flush(transcript)
 ** 2007-08-29:  Sopen0
+** 2008-02-21:  corrected typing error in isoutfile and isinpfile
 */
 
 #include "common.h"
@@ -318,7 +319,7 @@ int mode;   /* aTEXT or BINARY */
     int fmode;
 
     strmptr = STREAMPTR(strom);
-    fmode = strmptr-mode;
+    fmode = strmptr->mode;
 
     return((fmode & OUTSTREAM) && ((fmode & BINARY) == mode));
 }
@@ -331,7 +332,7 @@ int mode;   /* aTEXT or BINARY */
     int fmode;
 
     strmptr = STREAMPTR(strom);
-    fmode = strmptr-mode;
+    fmode = strmptr->mode;
 
     return((fmode & INSTREAM) && ((fmode & BINARY) == mode));
 }
