@@ -130,6 +130,30 @@ PRIVATE char *rBufptr;      /* pointer to char in rBuf */
 
 PRIVATE int inpsource = TERMINALINP;
 
+// #define SCANLOG
+#ifdef SCANLOG
+char *slogfile = "scan.log";
+FILE *Slog = NULL;
+
+int slognum(tok)
+int tok;
+{
+fprintf(Slog,"%d\n",tok);
+return tok;
+}
+int slogmess(mess)
+char *mess;
+{
+fprintf(Slog,"%s",mess);
+return strlen(mess);
+}
+int slogtick(ch)
+int ch;
+{
+fprintf(Slog,"%c",ch);
+return ch;
+}
+#endif
 /*--------------------------------------------------------------------*/
 PUBLIC void iniscan()
 {
@@ -301,7 +325,13 @@ int skip;   /* if skip != 0, skips EOL */
     int ch, res;
     ifun0 nextch;
 
+
     nextch = chread(strom);
+#ifdef SCANLOG
+if (Slog == NULL)
+Slog = fopen(slogfile,"w");
+slogmess("inside nexttok\n");
+#endif
   nochmal:
     ch = skipblanks(cStream->ch,nextch);
 
@@ -316,6 +346,10 @@ int skip;   /* if skip != 0, skips EOL */
     else if(isdecdigit(ch)) {
         StrBuf[0] = ch;
         res = numinp(nextch,StrBuf,strbufSize);
+#ifdef SCANLOG
+slogmess("Curnum[0] = ");
+slognum((int)Curnum.digits[0]);
+#endif
     }
     else switch(ch) {
     case '+':
@@ -483,6 +517,11 @@ int skip;   /* if skip != 0, skips EOL */
         buf[0] = nextch();
         res = bstrinp(nextch,buf,2*strbufSize);
         break;
+    case '\015':    /* CR */
+        ch = peekchar(nextch);
+        if (ch == EOL)
+            nextch();
+        /* fall through */
     case EOL:
     case FORMFEED:
         if(skip) {
@@ -492,20 +531,14 @@ int skip;   /* if skip != 0, skips EOL */
         else
             res = EOLTOK;
         break;
-#ifdef genUNiX
-    case '\015':    /* handle DOS line endings under UNiX */
-        nextch();
-        goto nochmal;
-        break;
-#endif
-	case ZESC:
-		ch = nextch();
-		if(ch == '\001') {
-			res = Z1TOK;
-			break;
-		}
-		else
-			goto nochmal;
+    case ZESC:
+        ch = nextch();
+        if(ch == '\001') {
+            res = Z1TOK;
+            break;
+        }
+        else
+            goto nochmal;
     case EOF:
         res = EOFTOK;
         break;
@@ -515,13 +548,20 @@ int skip;   /* if skip != 0, skips EOL */
         break;
     }
     cStream->tok = res;
+#ifdef SCANLOG
+slogmess("exiting nexttok, tok =\n");
+slognum(res);
+#endif
     return(res);
 }
 /*------------------------------------------------------------*/
 PUBLIC int curtok(strom)
 truc *strom;
 {
-    return(*STREAMTOKPTR(strom));
+    struct stream *strmptr;
+
+    strmptr = STREAMPTR(strom);
+    return(strmptr->tok);
 }
 /*------------------------------------------------------------*/
 PUBLIC int fltreadprec()    /* used by parser */
@@ -551,21 +591,22 @@ PRIVATE int skipblanks(ch,nextch)
 int ch;
 ifun0 nextch;
 {
-    while(ch == ' ' || ch == '\t')
+    while(ch == ' ' || ch == '\t') {
         ch = nextch();
+    }
     return(ch);
 }
 /*-----------------------------------------------------------------*/
 /*
 ** mode = 0: Loescht blanks vom Anfang des Strings str
 ** mode != 0: Loescht blanks vom Anfang des Strings und gibt
-**  dann den String, der aus der naechsten fortlaufenden Serie 
+**  dann den String, der aus der naechsten fortlaufenden Serie
 **  von Nicht-Blanks besteht, zurueck.
 ** Arbeitet destruktiv auf str !!
 */
 PUBLIC char *trimblanks(str,mode)
 char *str;
-int mode;   
+int mode;
 {
     char *str1;
 
@@ -591,9 +632,9 @@ truc *strom;
 
     nextch = chread(strom);
     tok = cStream->tok;
-    if(tok == EOLTOK) {
+    while(tok == EOLTOK) {
         nextch();
-        tok = nexttok(strom,1);
+        tok = nexttok(strom,0);
     }
     return(tok);
 }
@@ -855,7 +896,7 @@ char *str;
 }
 /*------------------------------------------------------------------*/
 /*
-** Liest aus dem aktuellen Eingabestrom in den Puffer buf 
+** Liest aus dem aktuellen Eingabestrom in den Puffer buf
 ** maximale fortlaufende Sequenz von Ziffern, wobei
 ** Zifferntrenner (globale Variable digsep) uebergangen werden.
 ** In buf[0] muss sich das aktuelle Zeichen befinden.
@@ -1080,7 +1121,7 @@ int maxlen;
 }
 /*-----------------------------------------------------------------*/
 /*
-** Benuetzt die globalen Variablen numstring, curbase, um den 
+** Benuetzt die globalen Variablen numstring, curbase, um den
 ** dadurch gegebenen (vorzeichenlosen) Integer in *numptr abzulegen
 ** Fuer numptr->digits wird AriBuf benuetzt,
 ** numptr->expo wird gleich 0 gesetzt
@@ -1094,7 +1135,7 @@ numdata *numptr;
 
     x = AriScratch;
     hilf = x + aribufSize;
-    
+
     switch(curbase) {
     case 10:
         n = str2big(numstring,x,hilf);
@@ -1141,7 +1182,7 @@ numdata *numptr;
 }
 /*-----------------------------------------------------------------*/
 /*
-** Benuetzt die globalen Variablen 
+** Benuetzt die globalen Variablen
 ** numstring, expstring, expsign
 ** um die dadurch gegebene Float-Zahl in *numptr abzulegen
 ** Fuer numptr->digits wird AriBuf benuetzt.
@@ -1152,7 +1193,7 @@ numdata *numptr;
     long decexp;
     word4 u;
     int prec = curfltprec + 2;
-    int len; 
+    int len;
     word2 *x, *hilf;
 
     x = AriScratch;
