@@ -20,12 +20,12 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 Address of the author
 
-	Otto Forster
-	Math. Institut der LMU
-	Theresienstr. 39
-	D-80333 Muenchen, Germany
+    Otto Forster
+    Math. Institut der LMU
+    Theresienstr. 39
+    D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email   forster@rz.mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -37,7 +37,7 @@ Email	forster@rz.mathematik.uni-muenchen.de
 **
 ** date of last change
 ** 1994-09-15
-** 1996-10-22	SysDUM
+** 1996-10-22   SysDUM
 ** 2002-04-27   datetime
 */
 
@@ -84,11 +84,11 @@ Email	forster@rz.mathematik.uni-muenchen.de
 #ifdef SCOUNiX
 #include <sys/types.h>
 #include <sys/timeb.h>
-#else	/* ifndef SCOUNiX */
+#else   /* ifndef SCOUNiX */
 #include <sys/time.h>
 static struct timezone t_z = {0,0};
 #endif
-#endif	/* TimeDUM */
+#endif  /* TimeDUM */
 #endif
 
 #ifndef DirDUM
@@ -115,13 +115,13 @@ int work_in[12],
 PUBLIC int VDI_handle;
 #endif
 
-PUBLIC void stacklimit	_((void));
-PUBLIC long stkcheck	_((void));
-PUBLIC long timer	    _((void));
-PUBLIC long datetime	_((int tim[6]));
-PUBLIC int sysrand	    _((void));
-PUBLIC void prologue	_((void));
-PUBLIC int epilogue	    _((void));
+PUBLIC void stacklimit  _((void));
+PUBLIC long stkcheck    _((void));
+PUBLIC long timer       _((void));
+PUBLIC long datetime    _((int tim[6]));
+PUBLIC int sysrand      _((void));
+PUBLIC void prologue    _((void));
+PUBLIC int epilogue     _((void));
 PUBLIC char *getworkdir   _((void));
 PUBLIC int setworkdir   _((char *pfad));
 
@@ -129,47 +129,47 @@ PUBLIC int setworkdir   _((char *pfad));
 PRIVATE long StackLimit;
 
 #ifdef MsDOS
-#define STACKLEN	50000	/* changed for MSVC, 98-04-12 */
+#define STACKLEN    50000   /* changed for MSVC, 98-04-12 */
 extern unsigned _stklen = STACKLEN;
 #endif
 
 #ifdef genUNiX
-#define STACKLEN	400000
+#define STACKLEN    400000
 #endif
 
 #ifdef MsWIN32
-#define STACKLEN	128000
+#define STACKLEN    128000
 #endif
 
 #ifdef DjGPP
-#define STACKLEN	128000
+#define STACKLEN    128000
 #endif
 
 /*-------------------------------------------------------------*/
 #ifdef ATARIST
 PUBLIC void stacklimit()
 {
-	extern long _StkLim;	/* im Startmodul TCSTSTK.O */
+    extern long _StkLim;    /* im Startmodul TCSTSTK.O */
 
-	StackLimit = _StkLim;
+    StackLimit = _StkLim;
 }
 #endif
 
 #ifdef MsDOS
 PUBLIC void stacklimit()
 {
-	char ptr;
+    char ptr;
 
-	StackLimit = (long)&ptr - (long)_stklen;
+    StackLimit = (long)&ptr - (long)_stklen;
 }
 #endif
 
 #if defined(UNiXorGCC) || defined(MsWIN32)
 PUBLIC void stacklimit()
 {
-	char ptr;
+    char ptr;
 
-	StackLimit = (long)&ptr - STACKLEN;
+    StackLimit = (long)&ptr - STACKLEN;
 }
 #endif
 /*-------------------------------------------------------------*/
@@ -180,23 +180,23 @@ PUBLIC void stacklimit()
 
 PUBLIC long stkcheck()
 {
-	long len;
+    long len;
 #ifdef StackDUM
-	len = 32000;
+    len = 32000;
 #else
-	extern long StackLimit;
-	char stkptr;
+    extern long StackLimit;
+    char stkptr;
 
-	len = (long)&stkptr - StackLimit;
+    len = (long)&stkptr - StackLimit;
 #endif
-	return(len);
+    return(len);
 }
 /*-------------------------------------------------------------*/
 #ifdef ATARIST
-#define TMULT	5
+#define TMULT   5
 #endif
 #ifdef MsDOS
-#define TMULT	54
+#define TMULT   54
 #endif
 #ifdef MsWIN32
 #define TMULT   1
@@ -205,9 +205,9 @@ PUBLIC long stkcheck()
 #if defined(DOSorTOS) || defined(MsWIN32)
 PUBLIC long timer()
 {
-	long t = clock();
-	t *= TMULT;
-	return(t);
+    long t = clock();
+    t *= TMULT;
+    return(t);
 }
 #endif
 
@@ -215,18 +215,18 @@ PUBLIC long timer()
 PUBLIC long timer()
 {
 #ifdef TimeDUM
-	return(0);
+    return(0);
 #else
 #ifdef SCOUNiX
-	struct timeb tb;
+    struct timeb tb;
 
-	ftime(&tb);
-	return(tb.time*1000 + tb.millitm);
+    ftime(&tb);
+    return(tb.time*1000 + tb.millitm);
 #else
-	struct timeval tv;
+    struct timeval tv;
 
-	gettimeofday(&tv,&t_z);
-	return(tv.tv_sec * 1000 + tv.tv_usec/1000);
+    gettimeofday(&tv,&t_z);
+    return(tv.tv_sec * 1000 + tv.tv_usec/1000);
 #endif
 #endif
 }
@@ -236,10 +236,11 @@ PUBLIC long datetime(tim)
 int tim[6];
 {
 #ifdef TimeDUM
+    int k;
     for(k=0; k<6; k++)
         tim[k] = 0;
     tim[2] = 1;
-	return(0);
+    return(0);
 #else
     time_t secs;
     struct tm *gmt;
@@ -267,9 +268,9 @@ PUBLIC int sysrand()
 #if defined(MsDOS) || defined(MsWIN32)
 PUBLIC int sysrand()
 {
-/*	randomize(); */ /* works with BORLAND C++ */
+/*  randomize(); */ /* works with BORLAND C++ */
     srand((unsigned)time(NULL));
-	return(rand());
+    return(rand());
 }
 #endif
 
@@ -277,18 +278,18 @@ PUBLIC int sysrand()
 PUBLIC int sysrand()
 {
 #ifdef SCOUNiX
-	struct timeb tb;
+    struct timeb tb;
 
-	ftime(&tb);
-	return(tb.time*1000 + tb.millitm);
+    ftime(&tb);
+    return(tb.time*1000 + tb.millitm);
 #else
 #ifndef TimeDUM
-	struct timeval tv;
+    struct timeval tv;
 
-	gettimeofday(&tv,&t_z);
-	return(tv.tv_usec);
+    gettimeofday(&tv,&t_z);
+    return(tv.tv_usec);
 #else
-	return timer() + 37421;
+    return timer() + 37421;
 #endif
 #endif
 }
@@ -342,42 +343,42 @@ PUBLIC void prologue()
 {
 #ifdef ATARIST
 
-	int i, handle;
-	int dum;
+    int i, handle;
+    int dum;
 
-	if(appl_init() < 0) {
-		puts("error in appl_init");
-		exit(1);
-	}
-	for(i=1; i<10; i++)
-		work_in[i] = 0;
-	work_in[10] = 2;
-	work_in[0] = graf_handle(&dum, &dum, &dum, &dum);
-	v_opnvwk(work_in, &handle, work_out);
-	if(handle <= 0) {
-		puts("error during program initialization");
-		exit(2);
-	}
-	v_hide_c(handle);
-	v_enter_cur(handle);
-	VDI_handle = handle;
+    if(appl_init() < 0) {
+        puts("error in appl_init");
+        exit(1);
+    }
+    for(i=1; i<10; i++)
+        work_in[i] = 0;
+    work_in[10] = 2;
+    work_in[0] = graf_handle(&dum, &dum, &dum, &dum);
+    v_opnvwk(work_in, &handle, work_out);
+    if(handle <= 0) {
+        puts("error during program initialization");
+        exit(2);
+    }
+    v_hide_c(handle);
+    v_enter_cur(handle);
+    VDI_handle = handle;
 #endif
 
 #ifdef DOSorUNiX
-	signal(SIGINT,ctrlcreset);
+    signal(SIGINT,ctrlcreset);
 #endif
-	return;
+    return;
 }
 /*------------------------------------------------------------------*/
 PUBLIC int epilogue()
 {
-	int ret = 0;
+    int ret = 0;
 #ifdef ATARIST
-	v_exit_cur(VDI_handle);
-	v_show_c(VDI_handle,1);
-	v_clsvwk(VDI_handle);
-	ret = appl_exit();
+    v_exit_cur(VDI_handle);
+    v_show_c(VDI_handle,1);
+    v_clsvwk(VDI_handle);
+    ret = appl_exit();
 #endif
-	return(ret);
+    return(ret);
 }
 /*****************************************************************/
